/// GraphicsTown2006 - CS559 Sample Code
// written by Michael Gleicher - originally written Fall, 2000
// updated Fall, 2005
// updated Fall, 2006
//
#include "GrTown_PCH.H"

#include "GrObject.H"
#include "FlyCamera.H"
#include "Behavior.H"
#include "GrWorld.H"
#include "DrawingState.H"

int noNameCtr=1;
GrObject::GrObject(char* nb, int& nn, char* xn) :
  ridable(0), interesting(0),
  children(0), parent(0)
{
  if (!nb) nb = "NoName";
  int cl = static_cast<int>(strlen(nb));
  if (xn)
	  cl += static_cast<int>(strlen(xn));
  name = new char[cl+12];
  if (xn)
	_snprintf(name,cl+11,"%s-%s-%d%c",nb,xn,nn++,0);
	else
	_snprintf(name,cl+11,"%s-%d%c",nb,nn++,0);

  idMatrix(transform);
  next = 0;
  behaviors = 0;

  rideAbove = 0;
}
GrObject::~GrObject()
{
}

// null functions - not abstract since not all objects will override
// them
void GrObject::draw(DrawingState* drst)
{
}
void GrObject::drawAfter(DrawingState* drst)
{
}


// for most objects, the way to use it as a camera is to take its
// transformation and invert it - potentially using "rideabove"
void GrObject::getCamera(Matrix camera)
{
  copyMatrix(transform,camera);
  invertMatrix(camera);
  camera[3][1] -= rideAbove;
}

// the user interface for the object may or may not do anything
bool GrObject::uiStep()
{
  return false;
}
int GrObject::handle(int)
{
	return 0;
}


// note - we might not really get to the time in one shot
// so let it take up to 10 tries
void GrObject::simulateUntil(unsigned long t)
{
  for(Behavior* b = behaviors; b; b=b->next) {
	  int ct=0;
	  while ((b->lastV < t) && (ct<10)) {
		b->simulateUntil(t);
		ct++;
	  }
	  /* if (ct>1) 
		  printf("needed %d simulation substeps\n",ct); */
	  // this is a common bug - you'll get caught here if you don't update lastV
	  if (ct>8)
		  printf("Warning! stuck behavior!\n");
  }
}

GrObject* GrObject::add(GrObject* g, float x, float y, float z, float ry)
{
  g->next = children;
  children = g;
  g->parent = this;
  Matrix t1,t2;
  rotMatrix(t1,'Y',ry);
  multMatrix(t1,g->transform,t2);
  copyMatrix(t2,g->transform);
  g->transform[3][0] += x;
  g->transform[3][1] += y;
  g->transform[3][2] += z;
  return g;
}


GrObject* add(GrObject* g, float x, float y, float z, float ry)
{
  g->next = theObjects;
  theObjects = g;
  Matrix t1,t2;
  rotMatrix(t1,'Y',ry);
  multMatrix(t1,g->transform,t2);
  copyMatrix(t2,g->transform);
  g->transform[3][0] += x;
  g->transform[3][1] += y;
  g->transform[3][2] += z;
  return g;
}

// draw a list of objects
void drawObList(GrObject* head, DrawingState* drst)
{
  glPushMatrix();
  for(GrObject* g = head; g; g = g->next) {
	  if (g != drst->camera) {
		  glPushMatrix();
		  glMultMatrixf(&g->transform[0][0]);
		  g->draw(drst);
		  glBindTexture(GL_TEXTURE_2D,0);
		  if (g->children) {
			  drawObList(g->children,drst);
		  }
		  glPopMatrix();
	  }
  }
  glPopMatrix();
}
void drawAfterObList(GrObject* head, DrawingState* drst)
{
  glPushMatrix();
  for(GrObject* g = head; g; g = g->next) {
	  if (g != drst->camera) {
		  glPushMatrix();
		  glMultMatrixf(&g->transform[0][0]);
		  g->drawAfter(drst);
		  glBindTexture(GL_TEXTURE_2D,0);
		  if (g->children) {
			  drawAfterObList(g->children,drst);
		  }
		  glPopMatrix();
	  }
  }
  glPopMatrix();
}


// $Header: /p/course/cs559-gleicher/private/CVS/GrTown/GrObject.cpp,v 1.2 2007/11/01 21:24:40 gleicher Exp $
